package com.xy.service.factory.device.impl.alipay;

import cn.hutool.json.JSONObject;
import com.xy.alipay.SpiDeviceService;
import com.xy.annotate.Factory;
import com.xy.config.DeviceThreadPoolConfig;
import com.xy.constants.SpiResponseConst;
import com.xy.consumer.connected.ConnectedMqttConfiguration;
import com.xy.consumer.connected.ConnectedProducer;
import com.xy.consumer.disconnect.DisconnectedMqttConfiguration;
import com.xy.consumer.disconnect.DisconnectedProducer;
import com.xy.device.EnumDeviceOnlineStatus;
import com.xy.dto.*;
import com.xy.dto.spi.DeviceAlarmNotifyDTO;
import com.xy.dto.spi.DeviceStatusChangeNotifyDTO;
import com.xy.entity.DeviceInfo;
import com.xy.entity.DeviceStatus;
import com.xy.service.AlipayDeviceService;
import com.xy.service.DeviceEventMsgServiceImpl;
import com.xy.service.DeviceInfoServiceImpl;
import com.xy.service.DeviceStatusServiceImpl;
import com.xy.service.factory.device.impl.open.OpenDeviceFactoryImpl;
import com.xy.service.factory.device.DeviceFactory;
import com.xy.utils.*;
import com.xy.utils.enums.DeviceErrorRecordTypesEnum;
import com.xy.vo.DeviceDetailVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * 支付宝开门柜设备 接口实现
 */
@Factory(type = "2")
@AllArgsConstructor
@Api(tags = "支付宝开门柜设备")
public class AliPayOpenDeviceFatoryImpl implements DeviceFactory, SpiDeviceService {

    private OpenDeviceFactoryImpl openDeviceFactoryImpl;

    private AlipayDeviceService alipayDeviceService;

    private ConnectedProducer connectedProducer;

    private DisconnectedProducer disconnectedProducer;

    private DeviceStatusServiceImpl deviceStatusService;

    private DeviceInfoServiceImpl deviceInfoService;

    private DeviceEventMsgServiceImpl deviceEventMsgService;

    @Override
    public R save(DeviceRegisterDto.Save save) {
        R r = openDeviceFactoryImpl.save(save);
        //支付宝设备注册
        if (r.getCode() == R.Enum.SUCCESS.getCode()) {
            alipayDeviceService.deviceAssign(new DeviceAssignDTO()
                    .setTerminalId(String.valueOf(save.getDeviceId()))
                    .setDeviceIdentifyType("DYNAMIC")
                    .setDeviceSn(save.getDeviceSn())
                    .setDeviceName(String.valueOf(save.getDeviceId()))
            );
        }
        return r;
    }

    @Override
    public R active(MercMiniDeviceDto.Active active) {
        return openDeviceFactoryImpl.active(active);
    }

    @Override
    public R<List<Tuple.Tuple3<Long, Long, Boolean>>> senCommand(List<CommandMqtt> commandMqtts) {
        commandMqtts.forEach(commandMqtt -> {
            JSONObject templet = commandMqtt.getTemplet();
            //重启设备
            if (templet.getStr("cmdType").equals("task")) {
                JSONObject data = templet.getJSONObject("data");
                if (data.getStr("type").equals("app") && data.getStr("task").equals("restart")) {
                    alipayDeviceService.deviceReboot(commandMqtt.getDeviceId().toString());
                }
            }
        });
        return R.ok();
    }

    @Override
    public R netRecord(DeviceNetRecordDto.Save save) {
        DeviceStatus deviceStatus = deviceStatusService.getById(save.getDeviceId());
        if (save.getNetStatus().equals(deviceStatus.getNetState())) {
            return R.ok();
        }
        Integer value = SysDictUtils.getValue(EnumDeviceOnlineStatus.Code.CODE.getCode(), EnumDeviceOnlineStatus.CONNECTED.getCode(), Integer.class);
        JSONObject jsonObject = new JSONObject().set("clientid", save.getDeviceId());
        if (save.getNetStatus().equals(value)) {
            connectedProducer.sendToMqtt(jsonObject.toString(), ConnectedMqttConfiguration.TOPIC, 1);
        } else {
            disconnectedProducer.sendToMqtt(jsonObject.toString(), DisconnectedMqttConfiguration.TOPIC, 1);
        }
        return R.ok();
    }

    @Override
    public R eventMsg(DeviceEventMsgDto.Save save) {
        return openDeviceFactoryImpl.eventMsg(save);
    }

    @Override
    public R busyStatus(MercMiniDeviceDto.BusySate busySate) {
        //支付宝设备运营状态修改为异步，业务方仅仅发送了修改通知，结果由支付宝异步通知
        DeviceChangeStatusDTO deviceChangeStatusDTO = new DeviceChangeStatusDTO()
                .setTerminalId(String.valueOf(busySate.getDeviceId()))
                .setOperatorId(AuthorizeUtils.getLoginId(String.class))
                .setStatus(busySate.getBusyState());
        alipayDeviceService.changeStatus(deviceChangeStatusDTO);
        return R.ok();
    }


    @Override
    @ApiOperation("设备告警通知SPI")
    public String deviceAlarmNotify(DeviceAlarmNotifyDTO deviceAlarmNotifyDTO) {
        //查询设备信息
        DeviceInfo deviceInfo = deviceInfoService.getById(Long.valueOf(deviceAlarmNotifyDTO.getTerminalId()));
        String faultCode = deviceAlarmNotifyDTO.getFaultCode();
        DeviceErrorRecordTypesEnum deviceErrorRecordTypesEnum = faultCode.equals("DeviceOffline") ? DeviceErrorRecordTypesEnum.NET
                : faultCode.equals("DoorOpenedOnNoTrade") ? DeviceErrorRecordTypesEnum.DOOR_LOCK
                : null;
        if (deviceErrorRecordTypesEnum == null) {
            return SpiResponseConst.SUCCESS;
        }
        //添加设备异常记录
        DeviceEventMsgDto.Save save = new DeviceEventMsgDto.Save()
                .setDeviceId(deviceInfo.getDeviceId());
        save.setCode(deviceErrorRecordTypesEnum.getCode());
        save.setMsg(deviceAlarmNotifyDTO.getFaultMsg());
        deviceEventMsgService.save(save);
        return SpiResponseConst.SUCCESS;
    }

    @Override
    @ApiOperation("设备状态变更通知SPI")
    public String deviceStatusChangeNotify(DeviceStatusChangeNotifyDTO deviceStatusChangeNotifyDTO) {
        //查询设备信息
        DeviceInfo deviceInfo = deviceInfoService.getById(Long.valueOf(deviceStatusChangeNotifyDTO.getTerminalId()));
        //冻结状态、运营状态
        FunctionUtils.NoParamsNoResult busyStatus = () -> {
            Integer status = deviceStatusChangeNotifyDTO.getStatus();
            DeviceInfo updateDeviceInfo = new DeviceInfo()
                    .setDeviceId(deviceInfo.getDeviceId())
                    .setBusyState(status)
                    .setFreezeStatus(status);
            deviceInfoService.updateById(updateDeviceInfo);
        };
        //联网状态
        FunctionUtils.NoParamsNoResult netRecord = () -> {
            Boolean online = deviceStatusChangeNotifyDTO.getOnline();
            if (online != null) {
                int netState = online ? SysDictUtils.getValue(EnumDeviceOnlineStatus.Code.CODE.getCode(), EnumDeviceOnlineStatus.CONNECTED.getCode(), Integer.class)
                        : SysDictUtils.getValue(EnumDeviceOnlineStatus.Code.CODE.getCode(), EnumDeviceOnlineStatus.DISCONNECT.getCode(), Integer.class);
                DeviceNetRecordDto.Save save = new DeviceNetRecordDto.Save()
                        .setDeviceId(deviceInfo.getDeviceId());
                save.setNetStatus(netState);
                netRecord(save);
            }
        };
        //激活状态
        FunctionUtils.NoParamsNoResult active = () -> {
            if (deviceStatusChangeNotifyDTO.getActiveStatus() != null && deviceStatusChangeNotifyDTO.getActiveStatus() == 1) {
                active(new MercMiniDeviceDto.Active().setDeviceId(deviceInfo.getDeviceId()));
            }
        };
        busyStatus.run();
        netRecord.run();
        active.run();
        return SpiResponseConst.SUCCESS;
    }

    @ApiOperation("刷新设备联网状态API")
    @PostMapping("refurbishDeviceNetWork")
    public R refurbishDeviceNetWork(@RequestBody List<Long> deviceIds) {
        ThreadPoolUtils.Execute execute = ThreadPoolUtils.excPoll(DeviceThreadPoolConfig.ALIPAY_DEVICE_DETAIL, deviceIds.size());
        deviceIds.forEach(deviceId -> execute.execute(() -> {
            DeviceDetailVO deviceDetailVO = alipayDeviceService.queryDetail(new DeviceDetailDTO().setTerminalId(String.valueOf(deviceId)));
            if (deviceDetailVO == null) {
                return;
            }
            DeviceNetRecordDto.Save save = new DeviceNetRecordDto.Save().setDeviceId(deviceId);
            save.setNetStatus(deviceDetailVO.getOnline()
                    ? SysDictUtils.getValue(EnumDeviceOnlineStatus.Code.CODE.getCode(), EnumDeviceOnlineStatus.CONNECTED.getCode(), Integer.class)
                    : SysDictUtils.getValue(EnumDeviceOnlineStatus.Code.CODE.getCode(), EnumDeviceOnlineStatus.DISCONNECT.getCode(), Integer.class));
            netRecord(save);
        }));
        execute.end();
        return R.ok();
    }
}
