package com.xy.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.xy.utils.PageBean;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotEmpty;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 设备计费表
 * </p>
 *
 * @author lijin
 * @since 2023-04-14
 */
public class DeviceChargingDto {

    @Data
    @Accessors(chain = true)
    public static class Charging {

        @NotEmpty(message = "设备id不能为空")
        @ApiModelProperty(value = "设备id集合")
        private List<Long> deviceIds;
    }

    @Data
    @Accessors(chain = true)
    public static class Page {

        @ApiModelProperty("分页对象")
        private PageBean page;

        @ApiModelProperty("商户id")
        private Long mercId;

        @ApiModelProperty("设备id")
        private Long deviceId;

        @ApiModelProperty("缴费状态 true=已缴费 false=欠费")
        private Boolean chargingStatus;

        @ApiModelProperty(value = "当前系统时间", hidden = true)
        private LocalDateTime thisTime = LocalDateTime.now();
    }

    @Data
    @Accessors(chain = true)
    public static class Vo {

        @ApiModelProperty(value = "设备id")
        private Long deviceId;

        @ApiModelProperty(value = "累计续费金额")
        private Integer chargingSumMoney;

        @ApiModelProperty(value = "试用剩余天")
        private Integer chargingX;

        @ApiModelProperty(value = "过期时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime timeout;

        @ApiModelProperty(value = "创建时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTime;

    }

    @Data
    @Accessors(chain = true)
    public static class PageVo {

        @ApiModelProperty(value = "计费信息")
        private Vo deviceCharging;

        @ApiModelProperty(value = "最新计费历史信息")
        private DeviceChargingHistoryDto.Vo deviceChargingHistory;

        @ApiModelProperty(value = "设备信息")
        private DeviceInfoDto.Vo deviceInfo;

        @ApiModelProperty(value = "计费标准金额/年")
        private Integer chargingMoney;

        @ApiModelProperty("缴费状态 true=已缴费 false=欠费")
        private Boolean chargingStatus;
    }

}
