package com.xy.service;

import com.xy.annotate.RestMappingController;
import com.xy.dto.DeviceInfoDto;
import com.xy.dto.DeviceSysinfoDto;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 设备-信息 服务类
 * </p>
 *
 * @author lijin
 * @since 2022-12-23
 */
@RestMappingController("device-info")
public interface DeviceInfoService {

    /**
     * 查询对象
     *
     * @param obj
     * @return
     */
    @PostMapping("obj")
    R<DeviceInfoDto.Vo> obj(@RequestBody DeviceInfoDto.Obj obj);

    /**
     * 设备访问历史添加
     *
     * @param obj
     * @return
     */
    @PostMapping("history")
    R history(@RequestBody DeviceInfoDto.Obj obj);

    /**
     * 开门检测
     *
     * @param obj
     * @return
     */
    @PostMapping("checkOpenDoor")
    R<DeviceInfoDto.Vo> checkOpenDoor(@RequestBody DeviceInfoDto.Obj obj);


    /**
     * 商户设备授权
     *
     * @param auth
     * @return {@link R}<{@link Boolean}>
     */
    @PostMapping("mercDeviceAuth")
    R<Boolean> mercDeviceAuth(@RequestBody @Validated DeviceInfoDto.MercDeviceAuthDto auth);

    @PostMapping("unBindMercDevice")
    R<Boolean> unBindMercDevice(@RequestBody @Validated DeviceInfoDto.MercDeviceUnBindDto dto);


    /**
     * 集合查询
     *
     * @param dto
     * @return
     */
    @PostMapping("list")
    R<List<DeviceInfoDto.Vo>> list(@RequestBody @Validated DeviceInfoDto.ListDto dto);

    /**
     * 通用列表查询
     *
     * @param dto
     * @return
     */
    @PostMapping("listCommon")
    R<List<DeviceInfoDto.Vo>> listCommon(@RequestBody @Validated DeviceInfoDto.ListCommon dto);

    /**
     * 更新商户线路
     *
     * @param updateLine
     * @return
     */
    @PostMapping("updateLine")
    R updateLine(@RequestBody @Validated DeviceInfoDto.UpdateLine updateLine);

    @PostMapping("pageSingle")
    R<PageBean<DeviceInfoDto.Vo>> pageSingle(@RequestBody DeviceInfoDto.PageSingle dto);

    /**
     * 更新商户点位
     *
     * @param updatePlace
     * @return
     */
    @PostMapping("updatePlace")
    R updatePlace(@RequestBody @Validated DeviceInfoDto.UpdatePlace updatePlace);


    /**
     * 商户设备算法列表
     *
     * @param dto
     * @return
     */
    @PostMapping("mercAlgorithmIdList")
    R<List<Long>> mercAlgorithmIdList(@RequestBody DeviceInfoDto.MercAlgorithmIdListDto dto);


    /**
     * 小程序-商户设备首页统计
     *
     * @param dto
     * @return
     */
    @PostMapping("mercHomeStatistical")
    R<DeviceInfoDto.MercHomeStatisticalVO> mercHomeStatistical(@RequestBody @Validated DeviceInfoDto.MercHomeQueryDTO dto);

    /**
     * 小程序-商户设备首页列表
     *
     * @param dto
     * @return
     */
    @PostMapping("mercHomeList")
    R<List<DeviceInfoDto.MercHomeListVO>> mercHomeList(@RequestBody @Validated DeviceInfoDto.MercHomeQueryDTO dto);


    /**
     * 小程序商户设备搜索
     *
     * @param page
     * @return
     */
    @PostMapping("mercDeviceSearchPage")
    R<PageBean<DeviceInfoDto.MerHomeSearchVO>> mercDeviceSearchPage(@RequestBody @Validated DeviceInfoDto.Page page);

    /**
     * 小程序商户设备详情-数据统计
     *
     * @param dto
     * @return
     */
    @PostMapping("dataCount")
    R<DeviceInfoDto.DeviceDataCountVO> dataCount(@RequestBody @Validated DeviceInfoDto.DeviceDataCountDTO dto);

    /**
     * 反显设别名称
     *
     * @param dto dto
     * @return {@link R}<{@link Map}<{@link Long},{@link String}>>
     */
    @PostMapping("getDeviceNameList")
    R<Map<Long, String>> getDeviceNameList(@RequestBody @Validated DeviceInfoDto.DeviceIdDto dto);


    /**
     * 查询设备信息
     *
     * @param dto
     * @return
     */
    @PostMapping("getDeviceSysinfo")
    R<DeviceSysinfoDto.Vo> getDeviceSysinfo(@RequestBody @Validated DeviceSysinfoDto.DeviceSysInfo dto);

}
